@@ -38,6 +38,8 @@ import warnings
 import copy
 import inspect
 from numbers import Number
+import numpy as np
+from sklearn.base import clone
 
 from entmoot.acquisition import _gaussian_acquisition
 
@@ -67,9 +69,6 @@ class Optimizer(object):
         - an instance of a `Dimension` object (`Real`, `Integer` or
           `Categorical`).
 
-        *NOTE: `Integer` and `Categorical` variables are not yet supported and
-        will be added in the next release.
-
     base_estimator : string, default: "GBRT",
         A default LightGBM surrogate model of the corresponding type is used  
         minimize `func`.
@@ -231,6 +230,26 @@ class Optimizer(object):
         self._n_initial_points = n_initial_points
         self.n_initial_points_ = n_initial_points
 
+        # Configure search space
+        
+        # initialize search space
+        import numpy as np
+        from entmoot.space.space import Space
+        from entmoot.space.space import Categorical
+
+        self.space = Space(dimensions)
+
+        self._initial_samples = None
+        self._initial_point_generator = cook_initial_point_generator(
+            initial_point_generator)
+
+        if self._initial_point_generator is not None:
+            transformer = self.space.get_transformer()
+            self._initial_samples = self._initial_point_generator.generate(
+                self.space.dimensions, n_initial_points,
+                random_state=self.rng.randint(0, np.iinfo(np.int32).max))
+            self.space.set_transformer(transformer)
+
         # Configure std estimator
 
         self.std_estimator = std_estimator
@@ -243,12 +262,14 @@ class Optimizer(object):
             raise ValueError("expected std_estimator to be in %s, got %s" %
                              (",".join(allowed_std_est), self.std_estimator))
 
+        self.scaled = self.acq_func_kwargs.get("scaled", False)
+
         # build std_estimator
         import numpy as np
         est_random_state = self.rng.randint(0, np.iinfo(np.int32).max)
         self.std_estimator = cook_std_estimator(
             std_estimator,
-            space=dimensions,
+            space=self.space,
             random_state=est_random_state,
             std_estimator_params=std_estimator_kwargs)
 
@@ -266,7 +287,7 @@ class Optimizer(object):
         base_estimator = cook_estimator(
             base_estimator, 
             self.std_estimator, 
-            space=dimensions,
+            space=self.space,
             random_state=est_random_state,
             base_estimator_params=base_estimator_kwargs)
 
@@ -287,28 +308,10 @@ class Optimizer(object):
 
         self.n_points = acq_optimizer_kwargs.get("n_points", 10000)
 
+        self.gurobi_env = acq_optimizer_kwargs.get("env", None)
+
         self.gurobi_timelimit = acq_optimizer_kwargs.get("gurobi_timelimit", None)
 
-        # Configure search space
-        
-        # initialize search space
-        import numpy as np
-        from entmoot.space.space import Space
-        from entmoot.space.space import Categorical
-
-        self.space = Space(dimensions)
-
-        self._initial_samples = None
-        self._initial_point_generator = cook_initial_point_generator(
-            initial_point_generator)
-
-        if self._initial_point_generator is not None:
-            transformer = self.space.get_transformer()
-            self._initial_samples = self._initial_point_generator.generate(
-                self.space.dimensions, n_initial_points,
-                random_state=self.rng.randint(0, np.iinfo(np.int32).max))
-            self.space.set_transformer(transformer)
-
         # Initialize storage for optimization
         if not isinstance(model_queue_size, (int, type(None))):
             raise TypeError("model_queue_size should be an int or None, "
@@ -597,18 +600,18 @@ class Optimizer(object):
                              "not compatible." % (type(x), type(y)))
 
         if self.models:
+            next_xt = self.space.transform([self._next_x])[0]
             temp_mu, temp_std = \
                 self.models[-1].predict(
-                    X=np.asarray(self._next_x).reshape(1, -1), 
-                    return_std=True)
+                    X=np.asarray(next_xt).reshape(1, -1), 
+                    return_std=True,
+                    scaled=self.scaled)
             self.model_mu.append(temp_mu)
             self.model_std.append(temp_std)
             
         # optimizer learned something new - discard cache
         self.cache_ = {}
 
-        from sklearn.base import clone
-
         # after being "told" n_initial_points we switch from sampling
         # random points to using a surrogate model
         if (fit and self._n_initial_points <= 0 and
@@ -680,7 +683,9 @@ class Optimizer(object):
                 add_model_core = \
                     self.acq_optimizer_kwargs.get("add_model_core", None)
                 gurobi_model = \
-                    get_core_gurobi_model(self.space, add_model_core)
+                    get_core_gurobi_model(
+                        self.space, add_model_core, env=self.gurobi_env
+                    )
 
                 if self.verbose == 2:
                     print("")
@@ -695,7 +700,9 @@ class Optimizer(object):
                 # and add to gurobi model
                 gbm_model_dict = {}
                 gbm_model_dict['first'] = est.get_gbm_model()
-                add_gbm_to_gurobi_model(gbm_model_dict, gurobi_model)
+                add_gbm_to_gurobi_model(
+                    self.space, gbm_model_dict, gurobi_model
+                )
 
                 # add std estimator to gurobi model
                 add_std_to_gurobi_model(est, gurobi_model)
@@ -722,9 +729,25 @@ class Optimizer(object):
                 # store optimality gap of gurobi computation
                 self.gurobi_mipgap.append(gurobi_model.mipgap)
 
-                next_x = np.asarray(
-                    [gurobi_model._c_x[i_x].x 
-                    for i_x in range(len(gurobi_model._c_x))])      
+                # output next_x
+                next_x = np.empty(len(self.space.dimensions))
+
+                # cont features
+                for i in gurobi_model._cont_var_dict.keys():
+                    next_x[i] = gurobi_model._cont_var_dict[i].x
+
+                # cat features
+                for i in gurobi_model._cat_var_dict.keys():
+                    cat = \
+                        [
+                            key 
+                            for key in gurobi_model._cat_var_dict[i].keys()
+                            if int(
+                                round(gurobi_model._cat_var_dict[i][key].x,1)
+                            ) == 1
+                        ]
+
+                    next_x[i] = cat[0]   
 
             # note the need for [0] at the end
             self._next_x = self.space.inverse_transform(
@@ -812,4 +835,51 @@ class Optimizer(object):
                                model_std=self.model_std,
                                gurobi_mipgap=self.gurobi_mipgap)
         result.specs = self.specs
-        return result
\ No newline at end of file
+        return result
+
+    def predict_with_est(self, x, return_std=True):
+        next_x = self.space.transform([x])[0]
+
+        if self.models:
+            temp_mu, temp_std = \
+                self.models[-1].predict(
+                    X=np.asarray(next_x).reshape(1, -1), 
+                    return_std=True,
+                    scaled=self.scaled)
+        else:
+            print("train first")
+            est = clone(self.base_estimator_)
+            est.fit(self.space.transform(self.Xi), self.yi)
+            temp_mu, temp_std = \
+                est.predict(
+                    X=np.asarray(next_x).reshape(1, -1), 
+                    return_std=True,
+                    scaled=self.scaled)
+        
+        if return_std:
+            return temp_mu[0], temp_std[0]
+        else:
+            return temp_mu[0]
+
+    def predict_with_acq(self, x):
+        next_x = self.space.transform([x])[0]
+
+        if self.models:
+            temp_val = _gaussian_acquisition(
+                X=np.asarray(next_x).reshape(1, -1), model=self.models[-1], 
+                y_opt=np.min(self.yi),
+                acq_func=self.acq_func,
+                acq_func_kwargs=self.acq_func_kwargs
+            )
+        else:
+            est = clone(self.base_estimator_)
+            est.fit(self.space.transform(self.Xi), self.yi)
+
+            temp_val = _gaussian_acquisition(
+                X=np.asarray(next_x).reshape(1, -1), model=est, 
+                y_opt=np.min(self.yi),
+                acq_func=self.acq_func,
+                acq_func_kwargs=self.acq_func_kwargs
+            )
+        
+        return temp_val[0]
\ No newline at end of file
